=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-11-23 01:28:47 +0000
+++ bzrlib/bzrdir.py	2006-11-29 03:43:35 +0000
@@ -50,6 +50,7 @@
     sha_strings,
     sha_string,
     )
+from bzrlib.smart.client import SmartClient
 from bzrlib.store.revision.text import TextRevisionStore
 from bzrlib.store.text import TextStore
 from bzrlib.store.versioned import WeaveStore
@@ -1476,7 +1477,14 @@
     easy to identify.
     """
 
-    def __init__(self, transport_server, transport_readonly_server, formats):
+    def __init__(self, vfs_factory, transport_server, transport_readonly_server,
+        formats):
+        """Create an object to adapt tests.
+
+        :param vfs_server: A factory to create a Transport Server which has
+            all the VFS methods working, and is writable.
+        """
+        self._vfs_factory = vfs_factory
         self._transport_server = transport_server
         self._transport_readonly_server = transport_readonly_server
         self._formats = formats
@@ -1485,6 +1493,7 @@
         result = unittest.TestSuite()
         for format in self._formats:
             new_test = deepcopy(test)
+            new_test.vfs_transport_factory = self._vfs_factory
             new_test.transport_server = self._transport_server
             new_test.transport_readonly_server = self._transport_readonly_server
             new_test.bzrdir_format = format
@@ -1978,6 +1987,15 @@
         else:
             return klass()
 
+    def initialize_on_transport(self, transport):
+        # hand off the request to the smart server
+        medium = transport.get_smart_medium()
+        client = SmartClient(medium)
+        path = client.remote_path_from_transport(transport)
+        response = SmartClient(medium).call('BzrDirFormat.initialize', path)
+        assert response[0] in ('ok', ), 'unexpected response code %s' % response[0]
+        return remote.RemoteBzrDir(transport)
+
     def _open(self, transport):
         return remote.RemoteBzrDir(transport)
 

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2006-11-28 02:21:24 +0000
+++ bzrlib/remote.py	2006-11-29 03:43:35 +0000
@@ -43,9 +43,10 @@
         # SMART_SERVER_MERGE_BLOCKER
         default_format = BzrDirFormat.get_default_format()
         self._real_bzrdir = default_format.open(transport, _found=True)
-        path = self._path_for_remote_call()
+        smartclient = client.SmartClient(self.client)
+        path = self._path_for_remote_call(smartclient)
         #self._real_bzrdir._format.probe_transport(transport)
-        response = client.SmartClient(self.client).call('probe_dont_use', path)
+        response = smartclient.call('probe_dont_use', path)
         if response == ('no',):
             raise errors.NotBranchError(path=transport.base)
         self._branch = None
@@ -66,8 +67,9 @@
 
     def open_branch(self, _unsupported=False):
         assert _unsupported == False, 'unsupported flag support not implemented yet.'
-        path = self._path_for_remote_call()
-        response = client.SmartClient(self.client).call('BzrDir.open_branch', path)
+        smartclient = client.SmartClient(self.client)
+        path = self._path_for_remote_call(smartclient)
+        response = smartclient.call('BzrDir.open_branch', path)
         assert response[0] == 'ok', 'unexpected response code %s' % response[0]
         if response[0] != 'ok':
             # this should probably be a regular translate no ?
@@ -91,9 +93,13 @@
             return format.open(self, _found=True, location=response[1])
 
     def open_repository(self):
-        path = self._path_for_remote_call()
-        response = client.SmartClient(self.client).call('BzrDir.find_repository', path)
-        assert response[0] == 'ok', 'unexpected response code %s' % response[0]
+        smartclient = client.SmartClient(self.client)
+        path = self._path_for_remote_call(smartclient)
+        response = smartclient.call('BzrDir.find_repository', path)
+        assert response[0] in ('ok', 'norepository'), \
+            'unexpected response code %s' % response[0]
+        if response[0] == 'norepository':
+            raise errors.NoRepositoryPresent(self)
         if response[1] == '':
             if vfs.vfs_enabled():
                 return RemoteRepository(self, self._real_bzrdir.open_repository())
@@ -105,9 +111,9 @@
     def open_workingtree(self):
         return RemoteWorkingTree(self, self._real_bzrdir.open_workingtree())
 
-    def _path_for_remote_call(self):
+    def _path_for_remote_call(self, client):
         """Return the path to be used for this bzrdir in a remote call."""
-        return unescape(urlparse(self.root_transport.base)[2])
+        return client.remote_path_from_transport(self.root_transport)
 
     def get_branch_transport(self, branch_format):
         return self._real_bzrdir.get_branch_transport(branch_format)
@@ -176,7 +182,7 @@
 
     def has_revision(self, revision_id):
         """See Repository.has_revision()."""
-        path = self.bzrdir._path_for_remote_call().encode('utf8')
+        path = self.bzrdir._path_for_remote_call(self._client)
         response = self._client.call('Repository.has_revision', path, revision_id.encode('utf8'))
         assert response[0] in ('ok', 'no'), 'unexpected response code %s' % response[0]
         return response[0] == 'ok'
@@ -229,7 +235,7 @@
         # XXX: TODO: this does not cache the revision history for the duration
         # of a lock, which is a bug - see the code for regular branches
         # for details.
-        path = self.bzrdir._path_for_remote_call()
+        path = self.bzrdir._path_for_remote_call(self._client)
         response = self._client.call2('Branch.revision_history', path)
         assert response[0][0] == 'ok', 'unexpected response code %s' % response[0]
         result = response[1].read_body_bytes().decode('utf8').split('\x00')

=== modified file 'bzrlib/smart/bzrdir.py'
--- bzrlib/smart/bzrdir.py	2006-11-22 03:46:04 +0000
+++ bzrlib/smart/bzrdir.py	2006-11-29 03:43:35 +0000
@@ -18,7 +18,7 @@
 
 
 from bzrlib import errors
-from bzrlib.bzrdir import BzrDir
+from bzrlib.bzrdir import BzrDir, BzrDirFormat
 from bzrlib.smart.request import SmartServerRequest, SmartServerResponse
 
 
@@ -49,6 +49,19 @@
             return SmartServerResponse(('norepository', ))
 
 
+class SmartServerRequestInitializeBzrDir(SmartServerRequest):
+
+    def do(self, path):
+        """Initialize a bzrdir at path.
+
+        The default format of the server is used.
+        :return: SmartServerResponse(('ok', ))
+        """
+        target_transport = self._backing_transport.clone(path)
+        BzrDirFormat.get_default_format().initialize_on_transport(target_transport)
+        return SmartServerResponse(('ok', ))
+
+
 class SmartServerRequestOpenBranch(SmartServerRequest):
 
     def do(self, path):

=== modified file 'bzrlib/smart/client.py'
--- bzrlib/smart/client.py	2006-11-24 03:41:36 +0000
+++ bzrlib/smart/client.py	2006-11-29 03:43:35 +0000
@@ -14,7 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from urlparse import urlparse
+
 from bzrlib.smart import protocol
+from bzrlib.urlutils import unescape
 
 
 class SmartClient(object):
@@ -41,3 +44,7 @@
         smart_protocol = protocol.SmartClientRequestProtocolOne(request)
         smart_protocol.call_with_body_bytes((method, ) + args, body)
         return smart_protocol.read_response_tuple()
+
+    def remote_path_from_transport(self, transport):
+        """Convert transport into a path suitable for using in a request."""
+        return unescape(urlparse(transport.base)[2]).encode('utf8')

=== modified file 'bzrlib/smart/request.py'
--- bzrlib/smart/request.py	2006-11-28 02:55:34 +0000
+++ bzrlib/smart/request.py	2006-11-29 03:43:35 +0000
@@ -238,6 +238,8 @@
 request_handlers.register_lazy(
     'BzrDir.find_repository', 'bzrlib.smart.bzrdir', 'SmartServerRequestFindRepository')
 request_handlers.register_lazy(
+    'BzrDirFormat.initialize', 'bzrlib.smart.bzrdir', 'SmartServerRequestInitializeBzrDir')
+request_handlers.register_lazy(
     'BzrDir.open_branch', 'bzrlib.smart.bzrdir', 'SmartServerRequestOpenBranch')
 request_handlers.register_lazy(
     'delete', 'bzrlib.smart.vfs', 'DeleteRequest')

=== modified file 'bzrlib/smart/server.py'
--- bzrlib/smart/server.py	2006-11-17 00:17:12 +0000
+++ bzrlib/smart/server.py	2006-11-29 03:43:35 +0000
@@ -100,35 +100,24 @@
 
 
 
-def SmartTCPServer_for_testing():
-    """Get a readwrite server for testing."""
-    return ReadWriteLocalSmartTCPServer(
-        transport.get_transport(urlutils.local_path_to_url('/')))
-
-
-def ReadonlySmartTCPServer_for_testing():
-    """Get a readonly server for testing."""
-    return ReadWriteLocalSmartTCPServer(
-        transport.get_transport('readonly+' + urlutils.local_path_to_url('/')))
-
-
-class ReadWriteLocalSmartTCPServer(SmartTCPServer):
+class SmartTCPServer_for_testing(SmartTCPServer):
     """Server suitable for use by transport tests.
     
     This server has a _homedir of the current cwd.
     """
 
-    def __init__(self, transport):
-        self._homedir = urlutils.local_path_to_url(os.getcwd())[7:]
-        # The server is set up by default like for ssh access: the client
-        # passes filesystem-absolute paths; therefore the server must look
-        # them up relative to the root directory.  it might be better to act
-        # a public server and have the server rewrite paths into the test
-        # directory.
-        SmartTCPServer.__init__(self, transport)
-        
-    def setUp(self):
+    def __init__(self):
+        # The server is set up by default like for inetd access: the backing
+        # transport is connected to a local path that is not '/'.
+        SmartTCPServer.__init__(self, None)
+
+    def get_backing_transport(self, backing_transport_server):
+        """Get a backing transport from a server we are decorating."""
+        return transport.get_transport('chroot+' + backing_transport_server.get_url())
+
+    def setUp(self, backing_transport_server):
         """Set up server for testing"""
+        self.backing_transport = self.get_backing_transport(backing_transport_server)
         self.start_background_thread()
 
     def tearDown(self):
@@ -137,10 +126,17 @@
     def get_url(self):
         """Return the url of the server"""
         host, port = self._server_socket.getsockname()
-        return "bzr://%s:%d%s" % (host, port, urlutils.escape(self._homedir))
+        return "bzr://%s:%d/" % (host, port)
 
     def get_bogus_url(self):
         """Return a URL which will fail to connect"""
         return 'bzr://127.0.0.1:1/'
 
 
+class ReadonlySmartTCPServer_for_testing(SmartTCPServer_for_testing):
+    """Get a readonly server for testing."""
+
+    def get_backing_transport(self, backing_transport_server):
+        """Get a backing transport from a server we are decorating."""
+        url = 'chroot+readonly+' + backing_transport_server.get_url()
+        return transport.get_transport(url)

=== modified file 'bzrlib/tests/HttpServer.py'
--- bzrlib/tests/HttpServer.py	2006-11-21 11:29:03 +0000
+++ bzrlib/tests/HttpServer.py	2006-11-29 03:43:35 +0000
@@ -277,8 +277,13 @@
         """Capture Server log output."""
         self.logs.append(format % args)
 
-    def setUp(self):
-        """See bzrlib.transport.Server.setUp."""
+    def setUp(self, decorated_transport):
+        """See bzrlib.transport.Server.setUp.
+        
+        :param decorated_transport: The transport that requests over this
+            protocol should be forwarded to. Note that this is currently not
+            supported for HTTP - it is ignored.
+        """
         self._home_dir = os.getcwdu()
         self._local_path_parts = self._home_dir.split(os.path.sep)
         self._http_starting = threading.Lock()

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-11-28 01:37:23 +0000
+++ bzrlib/tests/__init__.py	2006-11-29 03:43:35 +0000
@@ -1240,7 +1240,8 @@
         # execution. Variables that the parameteriser sets need to be 
         # ones that are not set by setUp, or setUp will trash them.
         super(TestCaseWithMemoryTransport, self).__init__(methodName)
-        self.transport_server = default_transport
+        self.vfs_transport_factory = default_transport
+        self.transport_server = None
         self.transport_readonly_server = None
 
     def failUnlessExists(self, path):
@@ -1284,10 +1285,10 @@
                 # readonly decorator requested
                 # bring up the server
                 self.__readonly_server = ReadonlyServer()
-                self.__readonly_server.setUp(self.get_server())
+                self.__readonly_server.setUp(self.get_vfs_only_server())
             else:
                 self.__readonly_server = self.create_transport_readonly_server()
-                self.__readonly_server.setUp()
+                self.__readonly_server.setUp(self.get_vfs_only_server())
             self.addCleanup(self.__readonly_server.tearDown)
         return self.__readonly_server
 
@@ -1306,8 +1307,8 @@
             base = base + relpath
         return base
 
-    def get_server(self):
-        """Get the read/write server instance.
+    def get_vfs_only_server(self):
+        """Get thae vfs only read/write server instance.
 
         This is useful for some tests with specific servers that need
         diagnostics.
@@ -1315,13 +1316,32 @@
         For TestCaseWithMemoryTransport this is always a MemoryServer, and there
         is no means to override it.
         """
+        if self.__vfs_server is None:
+            self.__vfs_server = MemoryServer()
+            self.__vfs_server.setUp()
+            self.addCleanup(self.__vfs_server.tearDown)
+        return self.__vfs_server
+
+    def get_server(self):
+        """Get the read/write server instance.
+
+        This is useful for some tests with specific servers that need
+        diagnostics.
+
+        This is built from the self.transport_server factory. If that is None,
+        then the self.get_vfs_server is returned.
+        """
         if self.__server is None:
-            self.__server = MemoryServer()
-            self.__server.setUp()
+            if self.transport_server is None:
+                return self.get_vfs_only_server()
+            else:
+                # bring up a decorated means of access to the vfs only server.
+                self.__server = self.transport_server()
+                self.__server.setUp(self.get_vfs_only_server())
             self.addCleanup(self.__server.tearDown)
         return self.__server
 
-    def get_url(self, relpath=None):
+    def _adjust_url(self, base, relpath):
         """Get a URL (or maybe a path) for the readwrite transport.
 
         This will either be backed by '.' or to an equivalent non-file based
@@ -1329,7 +1349,6 @@
         relpath provides for clients to get a path relative to the base url.
         These should only be downwards relative, not upwards.
         """
-        base = self.get_server().get_url()
         if relpath is not None and relpath != '.':
             if not base.endswith('/'):
                 base = base + '/'
@@ -1343,6 +1362,27 @@
                 base += urlutils.escape(relpath)
         return base
 
+    def get_url(self, relpath=None):
+        """Get a URL (or maybe a path) for the readwrite transport.
+
+        This will either be backed by '.' or to an equivalent non-file based
+        facility.
+        relpath provides for clients to get a path relative to the base url.
+        These should only be downwards relative, not upwards.
+        """
+        base = self.get_server().get_url()
+        return self._adjust_url(base, relpath)
+
+    def get_vfs_only_url(self, relpath=None):
+        """Get a URL (or maybe a path for the plain old vfs transport.
+
+        This will never be a smart protocol.
+        :param relpath: provides for clients to get a path relative to the base
+            url.  These should only be downwards relative, not upwards.
+        """
+        base = self.get_vfs_only_server().get_url()
+        return self._adjust_url(base, relpath)
+
     def _make_test_root(self):
         if TestCaseWithMemoryTransport.TEST_ROOT is not None:
             return
@@ -1545,24 +1585,17 @@
     readwrite one must both define get_url() as resolving to os.getcwd().
     """
 
-    def create_transport_server(self):
-        """Create a transport server from class defined at init.
-
-        This is mostly a hook for daughter classes.
-        """
-        return self.transport_server()
-
-    def get_server(self):
+    def get_vfs_only_server(self):
         """See TestCaseWithMemoryTransport.
 
         This is useful for some tests with specific servers that need
         diagnostics.
         """
-        if self.__server is None:
-            self.__server = self.create_transport_server()
-            self.__server.setUp()
-            self.addCleanup(self.__server.tearDown)
-        return self.__server
+        if self.__vfs_server is None:
+            self.__vfs_server = self.vfs_transport_factory()
+            self.__vfs_server.setUp()
+            self.addCleanup(self.__vfs_server.tearDown)
+        return self.__vfs_server
 
     def make_branch_and_tree(self, relpath, format=None):
         """Create a branch on the transport and a tree locally.
@@ -1613,7 +1646,7 @@
 
     def setUp(self):
         super(TestCaseWithTransport, self).setUp()
-        self.__server = None
+        self.__vfs_server = None
 
 
 class ChrootedTestCase(TestCaseWithTransport):
@@ -1630,7 +1663,9 @@
 
     def setUp(self):
         super(ChrootedTestCase, self).setUp()
-        if not self.transport_server == MemoryServer:
+        # NB: one HttpServer is taught to decorate properly,
+        # this hack can be removed.
+        if not self.vfs_transport_factory == MemoryServer:
             self.transport_readonly_server = HttpServer
 
 

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2006-11-28 01:37:23 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2006-11-29 03:43:35 +0000
@@ -444,7 +444,7 @@
 
     def setUp(self):
         super(ChrootedTests, self).setUp()
-        if not self.transport_server == MemoryServer:
+        if not self.vfs_transport_factory == MemoryServer:
             self.transport_readonly_server = HttpServer
 
     def test_open_containing(self):

=== modified file 'bzrlib/tests/bzrdir_implementations/__init__.py'
--- bzrlib/tests/bzrdir_implementations/__init__.py	2006-11-17 00:17:12 +0000
+++ bzrlib/tests/bzrdir_implementations/__init__.py	2006-11-29 03:43:35 +0000
@@ -31,6 +31,7 @@
                           TestLoader,
                           TestSuite,
                           )
+from bzrlib.transport.memory import MemoryServer
 
 
 def test_suite():
@@ -41,6 +42,7 @@
     formats = BzrDirFormat.known_formats()
     adapter = BzrDirTestProviderAdapter(
         default_transport,
+        None,
         # None here will cause a readonly decorator to be created
         # by the TestCaseWithTransport.get_readonly_transport method.
         None,
@@ -55,12 +57,13 @@
     from bzrlib.smart.server import SmartTCPServer_for_testing, ReadonlySmartTCPServer_for_testing
     from bzrlib.remote import RemoteBzrDirFormat
 
-    transport_server = SmartTCPServer_for_testing
+    # test the remote server behaviour using a MemoryTransport
     smart_server_suite = TestSuite()
     adapt_to_smart_server = BzrDirTestProviderAdapter(
-            transport_server,
-            ReadonlySmartTCPServer_for_testing,
-            [(RemoteBzrDirFormat())])
+        MemoryServer,
+        SmartTCPServer_for_testing,
+        ReadonlySmartTCPServer_for_testing,
+        [(RemoteBzrDirFormat())])
     adapt_modules(test_bzrdir_implementations,
                   adapt_to_smart_server,
                   TestLoader(),

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-11-22 02:49:26 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-11-29 03:43:35 +0000
@@ -1452,9 +1452,11 @@
             # they may not be initializable.
             return
         # supported formats must be able to init and open
-        url = self.get_url('subdir')
-        get_transport(self.get_url()).mkdir('subdir')
-        made_control = self.bzrdir_format.initialize(url)
+        # - do the vfs initialisation over the basic vfs transport
+        # XXX: TODO this should become a 'bzrdirlocation' api call.
+        url = self.get_vfs_only_url('subdir')
+        get_transport(self.get_vfs_only_url()).mkdir('subdir')
+        made_control = self.bzrdir_format.initialize(self.get_url('subdir'))
         try:
             repo = made_control.open_repository()
             # if there is a repository, then the format cannot ever hit this 
@@ -1462,7 +1464,7 @@
             return
         except errors.NoRepositoryPresent:
             pass
-        opened_control = bzrdir.BzrDir.open(self.get_readonly_url('subdir'))
+        made_control = bzrdir.BzrDir.open(self.get_readonly_url('subdir'))
         self.assertRaises(errors.NoRepositoryPresent,
-                          opened_control.find_repository)
+                          made_control.find_repository)
 

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2006-10-17 14:01:12 +0000
+++ bzrlib/tests/test_bzrdir.py	2006-11-29 03:43:35 +0000
@@ -266,7 +266,7 @@
 
     def test_create_branch_convenience_root(self):
         """Creating a branch at the root of a fs should work."""
-        self.transport_server = MemoryServer
+        self.vfs_transport_factory = MemoryServer
         # outside a repo the default convenience output is a repo+branch_tree
         old_format = bzrdir.BzrDirFormat.get_default_format()
         bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
@@ -365,7 +365,7 @@
 
     def setUp(self):
         super(ChrootedTests, self).setUp()
-        if not self.transport_server == MemoryServer:
+        if not self.vfs_transport_factory == MemoryServer:
             self.transport_readonly_server = HttpServer
 
     def test_open_containing(self):
@@ -581,7 +581,7 @@
 
     def setUp(self):
         super(NonLocalTests, self).setUp()
-        self.transport_server = MemoryServer
+        self.vfs_transport_factory = MemoryServer
     
     def test_create_branch_convenience(self):
         # outside a repo the default convenience output is a repo+branch_tree

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2006-11-28 02:21:24 +0000
+++ bzrlib/tests/test_remote.py	2006-11-29 03:43:35 +0000
@@ -30,14 +30,12 @@
 from bzrlib.branch import Branch
 
 
-class BasicRemoteObjectTests(tests.TestCaseInTempDir):
+class BasicRemoteObjectTests(tests.TestCaseWithTransport):
 
     def setUp(self):
-        tests.TestCaseInTempDir.setUp(self)
-        self.server = server.SmartTCPServer_for_testing()
-        self.server.setUp()
-        self.addCleanup(self.server.tearDown)
-        self.transport = remote_transport.SmartTCPTransport(self.server.get_url())
+        super(BasicRemoteObjectTests, self).setUp()
+        self.transport_server = server.SmartTCPServer_for_testing
+        self.transport = self.get_transport()
         self.client = self.transport.get_smart_client()
         # make a branch that can be opened over the smart transport
         self.local_wt = BzrDir.create_standalone_workingtree('.')

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-11-28 01:37:23 +0000
+++ bzrlib/tests/test_selftest.py	2006-11-29 03:43:35 +0000
@@ -187,17 +187,21 @@
         from bzrlib.bzrdir import BzrDirTestProviderAdapter
         input_test = TestBzrDirProviderAdapter(
             "test_adapted_tests")
+        vfs_factory = "v"
         server1 = "a"
         server2 = "b"
         formats = ["c", "d"]
-        adapter = BzrDirTestProviderAdapter(server1, server2, formats)
+        adapter = BzrDirTestProviderAdapter(vfs_factory,
+            server1, server2, formats)
         suite = adapter.adapt(input_test)
         tests = list(iter(suite))
         self.assertEqual(2, len(tests))
         self.assertEqual(tests[0].bzrdir_format, formats[0])
+        self.assertEqual(tests[0].vfs_transport_factory, vfs_factory)
         self.assertEqual(tests[0].transport_server, server1)
         self.assertEqual(tests[0].transport_readonly_server, server2)
         self.assertEqual(tests[1].bzrdir_format, formats[1])
+        self.assertEqual(tests[1].vfs_transport_factory, vfs_factory)
         self.assertEqual(tests[1].transport_server, server1)
         self.assertEqual(tests[1].transport_readonly_server, server2)
 
@@ -488,7 +492,7 @@
         from bzrlib.transport import get_transport
         from bzrlib.transport.memory import MemoryServer
         from bzrlib.transport.readonly import ReadonlyTransportDecorator
-        self.transport_server = MemoryServer
+        self.vfs_transport_factory = MemoryServer
         self.transport_readonly_server = None
         # calling get_readonly_transport() constructs a decorator on the url
         # for the server
@@ -530,7 +534,7 @@
 
     def setUp(self):
         super(TestTestCaseTransports, self).setUp()
-        self.transport_server = MemoryServer
+        self.vfs_transport_factory = MemoryServer
 
     def test_make_bzrdir_preserves_transport(self):
         t = self.get_transport()

=== modified file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	2006-11-28 01:37:23 +0000
+++ bzrlib/tests/test_sftp_transport.py	2006-11-29 03:43:35 +0000
@@ -42,9 +42,9 @@
     if getattr(testcase, '_get_remote_is_absolute', None) is None:
         testcase._get_remote_is_absolute = True
     if testcase._get_remote_is_absolute:
-        testcase.transport_server = SFTPAbsoluteServer
+        testcase.vfs_transport_factory = SFTPAbsoluteServer
     else:
-        testcase.transport_server = SFTPHomeDirServer
+        testcase.vfs_transport_factory = SFTPHomeDirServer
     testcase.transport_readonly_server = HttpServer
 
 
@@ -293,7 +293,7 @@
             server._vendor = self._test_vendor
             return server
         self._test_vendor = 'loopback'
-        self.transport_server = create_server
+        self.vfs_transport_server = create_server
         f = open('a_file', 'wb')
         try:
             f.write('foobar\n')

=== modified file 'bzrlib/tests/test_smart.py'
--- bzrlib/tests/test_smart.py	2006-11-28 02:55:34 +0000
+++ bzrlib/tests/test_smart.py	2006-11-29 03:43:35 +0000
@@ -16,7 +16,7 @@
 
 """Tests for the smart wire/domain protococl."""
 
-from bzrlib import errors, smart, tests
+from bzrlib import bzrdir, errors, smart, tests
 from bzrlib.smart.request import SmartServerResponse
 import bzrlib.smart.bzrdir
 import bzrlib.smart.branch
@@ -112,6 +112,37 @@
                 u'\xc8abc'.encode('utf8')))
 
 
+class TestSmartServerRequestInitializeBzrDir(tests.TestCaseWithTransport):
+
+    def test_empty_dir(self):
+        """Initializing an empty dir should succeed and do it."""
+        backing = self.get_transport()
+        request = smart.bzrdir.SmartServerRequestInitializeBzrDir(backing)
+        self.assertEqual(SmartServerResponse(('ok', )),
+            request.execute(backing.local_abspath('.')))
+        made_dir = bzrdir.BzrDir.open_from_transport(backing)
+        # no branch, tree or repository is expected with the current 
+        # default formart.
+        self.assertRaises(errors.NoWorkingTree, made_dir.open_workingtree)
+        self.assertRaises(errors.NotBranchError, made_dir.open_branch)
+        self.assertRaises(errors.NoRepositoryPresent, made_dir.open_repository)
+
+    def test_missing_dir(self):
+        """Initializing a missing directory should fail like the bzrdir api."""
+        backing = self.get_transport()
+        request = smart.bzrdir.SmartServerRequestInitializeBzrDir(backing)
+        self.assertRaises(errors.NoSuchFile,
+            request.execute, backing.local_abspath('subdir'))
+
+    def test_initialized_dir(self):
+        """Initializing an extant bzrdir should fail like the bzrdir api."""
+        backing = self.get_transport()
+        request = smart.bzrdir.SmartServerRequestInitializeBzrDir(backing)
+        self.make_bzrdir('subdir')
+        self.assertRaises(errors.FileExists,
+            request.execute, backing.local_abspath('subdir'))
+
+
 class TestSmartServerRequestOpenBranch(tests.TestCaseWithTransport):
 
     def test_no_branch(self):
@@ -198,6 +229,9 @@
             smart.request.request_handlers.get('BzrDir.find_repository'),
             smart.bzrdir.SmartServerRequestFindRepository)
         self.assertEqual(
+            smart.request.request_handlers.get('BzrDirFormat.initialize'),
+            smart.bzrdir.SmartServerRequestInitializeBzrDir)
+        self.assertEqual(
             smart.request.request_handlers.get('BzrDir.open_branch'),
             smart.bzrdir.SmartServerRequestOpenBranch)
         self.assertEqual(

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2006-11-28 01:37:23 +0000
+++ bzrlib/tests/test_smart_transport.py	2006-11-29 03:43:35 +0000
@@ -1477,13 +1477,9 @@
         # The 'readv' command in the smart protocol both sends and receives bulk
         # data, so we use that.
         self.build_tree(['data-file'])
-        http_server = HTTPServerWithSmarts()
-        http_server._url_protocol = url_protocol
-        http_server.setUp()
-        self.addCleanup(http_server.tearDown)
-
-        http_transport = get_transport(http_server.get_url())
-
+        self.transport_readonly_server = HTTPServerWithSmarts
+
+        http_transport = self.get_readonly_transport()
         medium = http_transport.get_smart_medium()
         #remote_transport = RemoteTransport('fake_url', medium)
         remote_transport = remote.RemoteTransport('/', medium=medium)
@@ -1511,7 +1507,7 @@
     def _test_http_send_smart_request(self, url_protocol):
         http_server = HTTPServerWithSmarts()
         http_server._url_protocol = url_protocol
-        http_server.setUp()
+        http_server.setUp(self.get_vfs_only_server())
         self.addCleanup(http_server.tearDown)
 
         post_body = 'hello\n'
@@ -1533,23 +1529,19 @@
         self._test_http_send_smart_request('http+urllib')
 
     def test_http_server_with_smarts(self):
-        http_server = HTTPServerWithSmarts()
-        http_server.setUp()
-        self.addCleanup(http_server.tearDown)
+        self.transport_readonly_server = HTTPServerWithSmarts
 
         post_body = 'hello\n'
         expected_reply_body = 'ok\x011\n'
 
-        smart_server_url = http_server.get_url() + '.bzr/smart'
+        smart_server_url = self.get_readonly_url('.bzr/smart')
         reply = urllib2.urlopen(smart_server_url, post_body).read()
 
         self.assertEqual(expected_reply_body, reply)
 
     def test_smart_http_server_post_request_handler(self):
-        http_server = HTTPServerWithSmarts()
-        http_server.setUp()
-        self.addCleanup(http_server.tearDown)
-        httpd = http_server._get_httpd()
+        self.transport_readonly_server = HTTPServerWithSmarts
+        httpd = self.get_readonly_server()._get_httpd()
 
         socket = SampleSocket(
             'POST /.bzr/smart HTTP/1.0\r\n'

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2006-11-28 01:37:23 +0000
+++ bzrlib/tests/test_transport.py	2006-11-29 03:43:35 +0000
@@ -294,10 +294,14 @@
         from bzrlib.transport import chroot
         transport = chroot.ChrootTransportDecorator('chroot+memory:///pathA/')
         self.assertEqual('memory:///pathA/', transport.chroot_url)
-
+        self.assertEqual('/', transport.chroot_relative)
+        transport = chroot.ChrootTransportDecorator('chroot+memory:///pathA')
+        self.assertEqual('memory:///pathA/', transport.chroot_url)
+        self.assertEqual('/', transport.chroot_relative)
         transport = chroot.ChrootTransportDecorator(
             'chroot+memory:///path/B', chroot='memory:///path/')
         self.assertEqual('memory:///path/', transport.chroot_url)
+        self.assertEqual('/B/', transport.chroot_relative)
 
     def test_append_file(self):
         transport = get_transport('chroot+file:///foo/bar')
@@ -309,7 +313,21 @@
 
     def test_clone(self):
         transport = get_transport('chroot+file:///foo/bar')
-        self.assertRaises(PathNotChild, transport.clone, '/foo')
+        # relpath from root and root path are the same
+        relpath_cloned = transport.clone('foo')
+        abspath_cloned = transport.clone('/foo')
+        self.assertEqual(relpath_cloned.base, abspath_cloned.base)
+        self.assertEqual(relpath_cloned.chroot_url, abspath_cloned.chroot_url)
+        self.assertEqual(relpath_cloned.chroot_relative,
+            abspath_cloned.chroot_relative)
+        transport = transport.clone('subdir')
+        # clone preserves chroot_url and adjusts chroot_relative
+        self.assertEqual('file:///foo/bar/', transport.chroot_url)
+        self.assertEqual('/subdir/', transport.chroot_relative)
+        transport = transport.clone('/otherdir')
+        # clone preserves chroot_url and adjusts chroot_relative
+        self.assertEqual('file:///foo/bar/', transport.chroot_url)
+        self.assertEqual('/otherdir/', transport.chroot_relative)
 
     def test_delete(self):
         transport = get_transport('chroot+file:///foo/bar')

=== modified file 'bzrlib/transport/chroot.py'
--- bzrlib/transport/chroot.py	2006-10-12 06:09:28 +0000
+++ bzrlib/transport/chroot.py	2006-11-29 03:43:35 +0000
@@ -17,15 +17,17 @@
 """Implementation of Transport that prevents access to locations above a set
 root.
 """
+from urlparse import urlparse
 
 from bzrlib import errors, urlutils
+from bzrlib.osutils import pathjoin
 from bzrlib.transport.decorator import TransportDecorator, DecoratorServer
 
 
 class ChrootTransportDecorator(TransportDecorator):
     """A decorator that can convert any transport to be chrooted.
 
-    This is requested via the 'chrooted+' prefix to get_transport().
+    This is requested via the 'chroot+' prefix to get_transport().
     """
 
     def __init__(self, url, _decorated=None, chroot=None):
@@ -35,6 +37,7 @@
             self.chroot_url = self._decorated.base
         else:
             self.chroot_url = chroot
+        self.chroot_relative = '/' + self._decorated.base[len(self.chroot_url):]
 
     @classmethod
     def _get_url_prefix(self):
@@ -58,8 +61,17 @@
         return TransportDecorator.append_bytes(self, relpath, bytes, mode=mode)
 
     def clone(self, offset=None):
-        self._ensure_relpath_is_child(offset)
-        return TransportDecorator.clone(self, offset)
+        if offset is None: return self
+        # the new URL we want to clone to is
+        # self.chroot_url + an adjusted self.chroot_relative, with the leading
+        # / removed.
+        newrelative = pathjoin(self.chroot_relative, offset)
+        newabs = self.chroot_url + newrelative[1:]
+        # now split to get a abspath without scheme
+        parsed = urlparse(newabs)
+        decorated_clone = self._decorated.clone(parsed[2])
+        return ChrootTransportDecorator(self._get_url_prefix() + newabs,
+            decorated_clone, self.chroot_url)
 
     def delete(self, relpath):
         self._ensure_relpath_is_child(relpath)

